function hOut = fakelabel(varargin)
% FAKELABEL:
%     Takes any, all or none of the following inputs:
%     
%     AX:          a list of axes handles (defaults to all axes on current
%                  figure)
%     WHICHLABELS: a string consisting of one or more of the letters X, Y
%                  and T denoting whether to duplicate the xlabel, ylabel
%                  or title (defaults to 'XY').
%     CLEARANCE:   a scalar denoting the number of pixels between axis and
%                  label, or a vector of three such values for xlabels,
%                  ylabels and titles respectively (defaults to an
%                  intelligent guess).
% 	
%     Additional optional graphics property arguments, to override the
%     properties of the existing labels.
% 
%    FAKELABEL(..., 'OFF')  removes fake labels, reinstating the real ones.

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

spc = 4;
defaultXlabelMinClearance = 15;
defaultYlabelMinClearance = 27;

h = []; whichLabels = []; clearance = []; opts = {};
legalLetters = 'xyt';
removeFlag = 0;
while ~isempty(varargin)
	arg = varargin{1};
	varargin(1) = [];
	if ischar(arg) & all(ismember(lower(arg(:)'), legalLetters)) & isempty(whichLabels)
		whichLabels = unique(lower(arg(:)'));
	elseif all(ishandle(arg(:))) & isempty(h)
		h = double(arg(:));
	elseif isequal(lower(arg), 'off')
		removeFlag = 1;
	elseif ismember(prod(size(arg)), [1 2 3]) & isnumeric(arg) & isempty(clearance)
		clearance = double(arg);
	elseif isstr(arg) & rem(length(varargin), 2) == 1
		opts = [{arg} varargin];
		varargin = {};
	else
		error('invalid input - type HELP FAKELABEL')
	end
end
if removeFlag & isempty(whichLabels), whichLabels = legalLetters; end
if isempty(h), h = gcf; end
if isempty(whichLabels), whichLabels = 'xy'; end
if isempty(clearance), clearance = [nan nan]; end
if prod(size(clearance)) == 1, clearance = repmat(clearance, [1 3]); end
if prod(size(clearance)) == 2, clearance(end+1) = nan; end
if prod(size(clearance)) ~= 3, error('CLEARANCE must have 1, 2 or 3 elements'), end

if any(~ismember(whichLabels, legalLetters)), error(['WHICHLABELS must contain only the letters ''' upper(legalLetters) '''']), end
axlist = unique(findall(h, 'type', 'axes'));

copyprops = {'userdata', 'horizontalalignment', 'verticalalignment', 'rotation', 'interpreter', 'fontname', 'fontunits', 'fontsize', 'fontweight', 'fontangle', 'color'};

if nargout, hOut = []; end

for i = 1:length(axlist)
	ax = axlist(i);

	for j = 1:length(whichLabels)
		letter = whichLabels(j);
		tag = ['Fake' upper(letter) 'Label'];
		previous = findall(ax, 'tag', tag);
		delete(previous)
				
		switch letter
		case 'x'
			old = get(ax, 'xlabel');
			wrongSide = strcmp(get(ax, 'xaxislocation'), 'top');
			pos = [0.5 wrongSide 0]; criticalDim = 2;
		case 'y'
			old = get(ax, 'ylabel');
			wrongSide = strcmp(get(ax, 'yaxislocation'), 'right');
			pos = [wrongSide 0.5 0]; criticalDim = 1;
		case 't'
			old = get(ax, 'title');
			wrongSide = 1;
			pos = [0.5 1 0]; criticalDim = 2;
		end
		str = get(old, 'string');
		if removeFlag
			if ~ isempty(previous), set(old, 'visible', 'on'), end
		elseif ~isempty(str) & ~all(isspace(str))
					
			delta = clearance(letter==legalLetters);
			if isnan(delta)
				oldfu = get(ax, 'fontunits');
				set(ax, 'fontunits', 'pixels')
				delta = get(ax, 'fontsize') + 2;
				set(ax, 'fontunits', oldfu)
				
				switch letter
				case 'x'
					delta = delta + 2 * spc;
					delta = max(delta, defaultXlabelMinClearance);
				case 'y'
					ytl = get(ax, 'yticklabel');
					delta = delta * 0.75 * size(ytl, 2); % too rough - try the following cheat instead

					fprops = {'fontname', 'fontunits', 'fontsize', 'fontweight', 'fontangle'};
					fprops(2, :) = get(ax, fprops);
					fake = text(1, 1, ytl, 'units','pixels', fprops{:}, 'visible', 'off');
					delta = get(fake, 'extent'); delta = delta(3) + spc;
					delete(fake)
					
					delta = max(delta, defaultYlabelMinClearance);
				case 't'
					if strcmp(get(ax, 'xaxislocation'), 'top')
						delta = (delta + 2 * spc) * 2;
					else
						delta = 2 * spc;
					end
				end
			end
			copyprops(2, :) = get(old, copyprops(1, :));
		
			pos = translateaxescoords(ax, pos, 'normalized', 'pixels');
			pos(criticalDim) = pos(criticalDim) + delta * sign(wrongSide - 0.5);
			pos = translateaxescoords(ax, pos, 'pixels', 'normalized');
			
			new = text(pos(1), pos(2), str, 'parent', ax, 'units', 'normalized');
			set(new, copyprops{:}, opts{:}, 'handlevisibility', 'off', 'visible', 'on', 'tag', tag)
			set(old, 'visible', 'off')
			if nargout, hOut(end+1, 1) = new; end
		end
	end	
end

